unit dyndbedit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, DB, Forms, Controls, Graphics, Dialogs,
  StdCtrls, dbctrls, Buttons, ExtCtrls, DBGrids,
  { Für Zeos benötigt }
  ZConnection, ZDataset, ZSequence,
  { Wird für die Sprachübersetzung benötigt }
  DefaultTranslator, LCLTranslator;

type

  { TForm1 }

  TForm1 = class(TForm)
    BitBtnShowEnglish: TBitBtn;
    BitBtnSetFields: TBitBtn;
    BitBtnDeleteFields: TBitBtn;
    DataSourceAdress: TDataSource;
    DBGrid1: TDBGrid;
    DBNavigator1: TDBNavigator;
    PanelTop: TPanel;
    PanelBottom: TPanel;
    ScrollBox1: TScrollBox;
    ZConnection1: TZConnection;
    ZQueryAdress: TZQuery;
    ZSequenceAdress: TZSequence;
    procedure BitBtnSetFieldsClick(Sender: TObject);
    procedure BitBtnDeleteFieldsClick(Sender: TObject);
    procedure BitBtnShowEnglishClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Arrays für die Erzeugung von Labels und DBEdits }
    EditsArray: Array of TDBEdit;
    LabelsArray: Array of TLabel;
  public

  end;

{ Resourcestrings werden von i18n übernommen und können übersetzt werden }
resourcestring

    { ADRESS }
    AdrCLIENTID          = 'CLIENTID';
    AdrADRESSID          = 'ADRESSID';
    AdrLOOKUP            = 'LOOKUP';

    AdrSALUTATION        = 'SALUTATION';
    AdrTITLE             = 'TITLE';
    AdrFORENAME          = 'FORENAME';
    AdrMIDDLENAME        = 'MIDDLENAME';
    AdrSURNAME           = 'SURNAME';
    AdrADRESSLINE1       = 'ADRESSLINE1';
    AdrADRESSLINE1NUMBER = 'ADRESSLINE1NUMBER';
    AdrADRESSLINE2       = 'ADRESSLINE2';
    AdrPOSTCODE          = 'POSTCODE';
    AdrCITY              = 'CITY';
    AdrCOUNTRY           = 'COUNTRY';
    AdrCOMPANY           = 'COMPANY';
    AdrCOMPANYBRANCH     = 'COMPANYBRANCH';

    AdrFILEALIDADRESS    = 'FILEALIDADRESS';
    AdrFILEALIDCOMPANY   = 'FILEALIDCOMPANY';
    AdrUIDCOMPANY        = 'UIDCOMPANY';
    AdrUIDFILEAL         = 'UIDFILEAL';

    AdrJOBPOSITION       = 'JOBPOSITION';
    AdrADRESSTYPE1       = 'ADRESSTYPE1';
    AdrADRESSTYPE2       = 'ADRESSTYPE2';
    AdrSTATE             = 'STATE';
    AdrCATEGORY          = 'CATEGORY';
    AdrRESOURCEID        = 'RESOURCEID';

    AdrADRUSERFIELD0     = 'ADRUSERFIELD0';
    AdrADRUSERFIELD1     = 'ADRUSERFIELD1';
    AdrADRUSERFIELD2     = 'ADRUSERFIELD2';
    AdrADRUSERFIELD3     = 'ADRUSERFIELD3';
    AdrADRUSERFIELD4     = 'ADRUSERFIELD4';
    AdrADRUSERFIELD5     = 'ADRUSERFIELD5';
    AdrADRUSERFIELD6     = 'ADRUSERFIELD6';
    AdrADRUSERFIELD7     = 'ADRUSERFIELD7';
    AdrADRUSERFIELD8     = 'ADRUSERFIELD8';
    AdrADRUSERFIELD9     = 'ADRUSERFIELD9';
    AdrGEOPOSITION       = 'ADRGEOPOSITION';
    AdrADRESSCREATED     = 'ADRESSCREATED';
    AdrADRESSLASTCHANGED = 'ADRESSLASTCHANGED';
    AdrADRESSMEMO        = 'ADRESSMEMO';

var
  Form1: TForm1;

{ *** IMPLEMENTATION ******************************************************* }
implementation

{$R *.lfm}

{ TForm1 }


{ BitBtnSetFieldsClick: Felder anzeigen }
procedure TForm1.BitBtnSetFieldsClick(Sender: TObject);
var
  i : Integer;
  s: string;
begin

  { Wenn die Felder schon produziert wurden nicht noch einmal produzieren    }
  { Normalerweise sollte das mit Enabled/Disabled der Buttons gemacht werden }
  if ((High(EditsArray)>0)) then exit;

  { ScrollBox unsichtbar beschleunigt den Aufbau }
  ScrollBox1.Visible:= false;

  { Adresstabelle öffnen, falls nicht geöffnet }
  if (not ZQueryAdress.Active) then ZQueryAdress.Open;

    with ZQueryAdress do begin
      { Anzahl Felder ermitteln }
      i:= ZQueryAdress.FieldCount;

      { Das Label- und DBEdits-Array für Anzahl Felder vorbereiten }
      SetLength(LabelsArray, i);
      SetLength(EditsArray, i);

      { Und jetzt in einer Schleife die einzelnen Label und DBEdit erstellen }
      for i:=0 to High(EditsArray) do begin

        { Das i-te Label erzeugen }
        LabelsArray[i] := TLabel.Create(self);

        { Dem erzeugten Label die Parameter zuweisen }
        with LabelsArray[i] do begin
          { Wo das Label positioniert wird }
          parent := ScrollBox1;
          top := i * 38;
          left := 30;
          width:= 240;
          height:= 32;

          { Farbeinstellung }
          color:= clYellow;

          { Fixe Größe, keine automatische Größenanpassung }
          autosize:= false;

          { Schrift und Schriftgröße }
          Font.Name:= 'Tahoma';
          Font.Size:= 14;

          { Der Name des Labels }
          name := 'label'+IntToStr(i);
          Alignment:= taRightJustify;

          { Feldname des aktuell bearbeitenden Feldes }
          s:= Fields[i].FieldName;

          { Feldname als Hint anzeigen }
          ShowHint:= True;
          Hint:= s;

          { Die jeweils eingestellte Landessprache "SetDefaultLang('de');" etc. zuweisen }
          { Nicht alle Datenfelder sind hier angeführt, bei den nicht übersetzten wird   }
          { die Feldbezeichnung angezeigt, da diese standardmäßig DisplayLabel sind      }
          case s of
            'ADRESSID':          s:= AdrADRESSID;

            'CLIENTID':          s:= AdrCLIENTID;
            'LOOKUP':            s:= AdrLOOKUP;

            'SALUTATION':        s:= AdrSALUTATION;
            'TITLE':             s:= AdrTITLE;
            'FORENAME':          s:= AdrFORENAME;
            'MIDDLENAME':        s:= AdrMIDDLENAME;
            'SURNAME':           s:= AdrSURNAME;
            'ADRESSLINE1':       s:= AdrADRESSLINE1;
            'ADRESSLINE1NUMBER': s:= AdrADRESSLINE1NUMBER;
            'ADRESSLINE2':       s:= AdrADRESSLINE2;
            'POSTCODE':          s:= AdrPOSTCODE;
            'CITY':              s:= AdrCITY;
            'COUNTRY':           s:= AdrCOUNTRY;
            'COMPANY':           s:= AdrCOMPANY;
            'COMPANYBRANCH':     s:= AdrCOMPANYBRANCH;

            'FILEALIDADRESS':    s:= AdrFILEALIDADRESS;
            'FILEALIDCOMPANY':   s:= AdrFILEALIDCOMPANY;
            'UIDCOMPANY':        s:= AdrUIDCOMPANY;
            'UIDFILEAL':         s:= AdrUIDFILEAL;

            'JOBPOSITION':       s:= AdrJOBPOSITION;
            'ADRESSTYPE1':       s:= AdrADRESSTYPE1;
            'ADRESSTYPE2':       s:= AdrADRESSTYPE2;
            'STATE':             s:= AdrSTATE;
            'CATEGORY':          s:= AdrCATEGORY;
            'RESOURCEID':        s:= AdrRESOURCEID;

            'ADRUSERFIELD0':     s:= AdrADRUSERFIELD0;
            'ADRUSERFIELD1':     s:= AdrADRUSERFIELD1;
            'ADRUSERFIELD2':     s:= AdrADRUSERFIELD2;
            'ADRUSERFIELD3':     s:= AdrADRUSERFIELD3;
            'ADRUSERFIELD4':     s:= AdrADRUSERFIELD4;
            'ADRUSERFIELD5':     s:= AdrADRUSERFIELD5;
            'ADRUSERFIELD6':     s:= AdrADRUSERFIELD6;
            'ADRUSERFIELD7':     s:= AdrADRUSERFIELD7;
            'ADRUSERFIELD8':     s:= AdrADRUSERFIELD8;
            'ADRUSERFIELD9':     s:= AdrADRUSERFIELD9;
            'GEOPOSITION':       s:= AdrGEOPOSITION;
            'ADRESSCREATED':     s:= AdrADRESSCREATED;
            'ADRESSLASTCHANGED': s:= AdrADRESSLASTCHANGED;
            'ADRESSMEMO':        s:= AdrADRESSMEMO;
          end;
          { Dem Label den Text der jeweiligen Landessprache zuweisen }
          Caption:= s+' ';

          { Mit dieser Zuweisung ERHÄLT in der Datenbank die Feldüberschrift eingedeutscht }
          { Fields[i].DisplayLabel erhält die Übersetzung des Feldbezeichnung              }
          { Fields[i].FieldName gibt aber immer den Feldnamen zurück                       }
          Fields[i].DisplayLabel:= s;

        end;

        { Das DBedit erzeugen }
        EditsArray[i] := TDBEdit.Create(self);

        { Den DBEdit die Parameter zuweisen }
        with EditsArray[i] do begin
          { Dem DBEdit die Position zuweisen }
          parent := ScrollBox1;
          top := i * 38;
          left := 300;
          height:= 32;
          width:= 600;

          { Farbe von DBEdit}
          color:= clForm;

          { Fixe Größe, keine automatische Größenanpassung }
          autosize:= false;

          { Schrift und Schriftgröße }
          Font.Name:= 'Tahoma';
          Font.Size:= 14;

          { Der Name der DBEdit Komponente }
          name := 'edit'+IntToStr(i);

          { Name des Datenfeldes für Zugriff zuweisen (DataSet) }
          DataField:= Fields[i].FieldName;
          DataSource:= DataSourceAdress;
        end;
      end;

      { Scrollbox nach erledigter Arbeit wieder anzeigen }
    ScrollBox1.Visible:= true;
  end;

  { Datenbankgrid erneut zuweisen, dann werden die deutschen Titel aus der }
  { Datenbank angezeigt, DBGrid vor und nach dem Aufruf anschauen          }
  DBGrid1.DataSource:= nil;
  DBGrid1.DataSource:= DataSourceAdress;
end;


{ BitBtnDeleteFieldsClick: Die angelegten Labels und DBEdits alle löschen  }
{ Dies löscht auch die Anzeige der Labels undDBEdits                       }
procedure TForm1.BitBtnDeleteFieldsClick(Sender: TObject);
var i: integer;
begin

  { Wieviele Elemente im Array enthalten sind , hier 44 Datenfelder }
  // i1:= Length(LabelsArray) -1;
  { Der Index des letzten Elements hier 43, 0..43 sind 44 Datenfelder }
  // i1:= High(EditsArray);


  { Nur löschen, wenn noch nicht gelöscht wurde }
  if ((High(EditsArray)>0)) then begin

    { Alle Komponenten löschen, diese könnten dann für eine andere Tabelle benutzt werden }
    { Wir verwenden High(EditsArray), da dann von 0..43 gelöscht wird, sind 44 Felder     }
    for i:=0 to High(EditsArray) do begin
      { Jedes einzelne Element löschen }
      EditsArray[i].Destroy;
      LabelsArray[i].Destroy;
    end;
    { Den Zähler der Arrayelemente auf 0 zurücksetzen }
    SetLength(EditsArray, 0);
    SetLength(LabelsArray, i);
  end;

end;

{ Zeigt alles auf Englisch }
procedure TForm1.BitBtnShowEnglishClick(Sender: TObject);
begin
  { Einstellen der englischen Sprache }
  SetDefaultLang('en');
  { Falls Felder angezeigt werden, loschen }
  BitBtnDeleteFields.Click;
  { Felder neu anzeigen }
  BitBtnSetFields.Click;
end;

{ FormActivate: Datenbank und Query für ADRESS öffnen                             }
procedure TForm1.FormActivate(Sender: TObject);
begin
  { Einstellen der deutschen Sprache }
  SetDefaultLang('de');

  with ZConnection1 do begin
    { Wenn das Turorial nicht unter "C:\ZEOS_DYNAMIC_DBEDIT\" abgespeichert wurde, }
    { dann müssen die Verbindungsdaten angepasst werden                            }
    Database:= 'C:\ZEOS_DYNAMIC_DBEDIT\COFFEENOW.FDB';
    LibraryLocation:= 'C:\ZEOS_DYNAMIC_DBEDIT\fb464\fbclient.dll';
    Connected:= true;
  end;
  { Query für Adresszugriff öffnen }
  ZQueryAdress.Open;
end;








end.

